#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <typeinfo>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "dialogs.h"
#include "projects.h"
#include "util.h"
#include "prefs.h"


void
on_new_proj_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *name, *dir;
	gint answer, type;
	 
	if(cur_project){
		answer = ask_yes_no_cancel("Project is opened. Do you want to save it?");
		switch(answer){
		case 0:cur_project->save();
			break;
		case 2:return;
			break;
		}; 
	};

	ask_new_project(name, dir, type);

	if(!name)
		return;
		
	if(CCppProject::create_empty(name,dir,
				     prefs->def_cppcmd.make,
				     prefs->def_cppcmd.compile,
				     prefs->def_cppcmd.debug,
				     prefs->def_cppcmd.execute)){
		g_free(dir);
		g_free(name);
		return;
	};
	
	if(cur_project)
		delete cur_project;
		
	cur_project = NULL;
	
	switch(type){
	case 1: cur_project = new CCppProject(name,dir,mainapp,mvpaned,c_list,o_list,e_list);
	};

	g_free(dir);
	g_free(name);
	
	up_interface();
}

void
on_open_proj_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gint answer;

	if(cur_project){
		answer = ask_yes_no_cancel("Project is opened. Do you want to save it?");
		switch(answer){
		case 0:cur_project->save();
			break;
		case 2:return;
			break;
		};

	};

	ask_open_project();
}

void
on_save_proj_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(cur_project){
		cur_project->save();
	};
}

void
on_save_proj_as_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	ask_saveas_project();
}

void
on_save_file_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	cur_project->cur_file->save();
}

void
on_save_file_as_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	ask_saveas_file();
}

void
on_save_all_files_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	cur_project->save_all_files();
}

void
on_close_file_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	cur_project->close_file();

	up_interface();
}

void
on_exit_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *msg;
	gint ans;
	
	if(cur_project){
		if(cur_project->has_modif_files()){
			msg = g_strdup("Project has modified files. Save them all?");
			ans = ask_yes_no_cancel(msg);
			g_free(msg);
			switch(ans){
			case 0:cur_project->save_all_files();break;
			case 2:return;
			};
		};
		msg = g_strdup("Project was modified. Save?");
		ans = ask_yes_no_cancel(msg);
		g_free(msg);
		switch(ans){
		case 0:cur_project->save();break;
		case 2:return;
		};
	};

	prefs->save();
	delete prefs;

	delete cur_project;
	gtk_main_quit();
}

void
on_cut_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	cur_project->cur_file->cut();
}

void
on_copy_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	cur_project->cur_file->copy();
}

void
on_paste_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	cur_project->cur_file->paste();
}

void
on_changelog_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar out_time[128], out[128];
	time_t ctime;
	time(&ctime);

	struct tm *t = localtime(&ctime);
	strftime(out_time,sizeof(out_time),"%a %b %d %X %Z %Y",t);

	g_snprintf(out,sizeof(out),"%s %s <%s>",out_time,prefs->name,prefs->mail);
	
	cur_project->cur_file->insert(out);
}

void
on_date_time_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar out[128];
	time_t ctime;
	time(&ctime);

	struct tm *t = localtime(&ctime);
	strftime(out,sizeof(out),"%a %b %d %X %Z %Y",t);
	
	cur_project->cur_file->insert(out);
}

void
on_name_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar out[128];

	g_snprintf(out,sizeof(out),"%s <%s>",prefs->name,prefs->mail);

	cur_project->cur_file->insert(out);
}

void
on_go_to_line_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gint ln = ask_line_number();
	if(ln)
		cur_project->cur_file->gotoline(ln);
}

void
on_find_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_replace_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_clear_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	cur_project->cur_file->clear();
}

void
on_add_file_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	ask_add_file();
}

void
on_remove_file_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *n = ask_remove_file();
	if(n){
		cur_project->remove_file(n);

		g_free(n);
	};
	up_interface();
}

void set_props(GtkWidget *btn, GtkWidget *prbox)
{
	cur_project->load(prbox);
}

void
on_properties_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *box = create_propbox();
	gtk_signal_connect(GTK_OBJECT(lookup_widget(box,"ok_btn")),"clicked",GTK_SIGNAL_FUNC(set_props),box);

	cur_project->save(box);
	gnome_dialog_run_and_close(GNOME_DIALOG(box));
}

void set_prefs(GtkWidget *btn, GtkWidget *prbox)
{
	prefs->load(prbox);
	prefs->save();
}

void
on_preferences_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *box = create_prefsbox();
	gtk_signal_connect(GTK_OBJECT(lookup_widget(box,"ok_btn")),"clicked",GTK_SIGNAL_FUNC(set_prefs),box);

	prefs->save(box);
	gnome_dialog_run_and_close(GNOME_DIALOG(box));
}

void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        GtkWidget *about = create_about();
	
	gnome_dialog_run_and_close(GNOME_DIALOG(about));
}

void
on_new_file_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *fullname;

	if(typeid(*cur_project) == typeid(CCppProject)){
		gchar *name_c, *name_h;
		gchar *msg;
		gint need_c, need_h;
		gint ans;
		
		ask_new_c_file(need_c,name_c,need_h,name_h);

		if(need_c&&cur_project->file_in_project(name_c)){
			msg = g_strconcat("File ",name_c," already in project. It wouldn't be added.",NULL);
			ask_error(msg);
			g_free(msg);
			g_free(name_c);
			need_c = FALSE;
		};

		if(need_h&&cur_project->file_in_project(name_h)){
			msg = g_strconcat("File ",name_h," already in project. It wouldn't be added.",NULL);
			ask_error(msg);
			g_free(msg);
			g_free(name_h);
			need_h = FALSE;
		};
	
		if(need_c){
			fullname = g_strconcat(cur_project->get_dir(),"/",name_c,NULL);
			if(file_exists(fullname)){
				msg = g_strconcat("File ",fullname," already exists. Overwrite?");
				ans = ask_yes_no(msg);
				g_free(msg);
				if(!ans)
					cur_project->new_file(name_c);
			}else
				cur_project->new_file(name_c);

			g_free(fullname);
			g_free(name_c);
		};

		if(need_h){
			fullname = g_strconcat(cur_project->get_dir(),"/",name_h,NULL);
			if(file_exists(fullname)){
				msg = g_strconcat("File ",fullname," already exists. Overwrite?");
				ans = ask_yes_no(msg);
				g_free(msg);
				if(!ans)
					cur_project->new_file(name_h);
			}else
				cur_project->new_file(name_h);

			g_free(fullname);
			g_free(name_h);
		};
	};
	
	up_interface();
}

void
proj_open_ok(GtkWidget *widget,GtkFileSelection *fs)
{
	gchar *name, *fname, *dir, *s, *s1;
	gchar *type;
	gint rev;
	
	name = g_strdup(gtk_file_selection_get_filename(fs));
	gtk_widget_destroy(GTK_WIDGET(fs));

	if(!file_exists(name)){
		ask_error("File not found!");
		g_free(name);
		return;
	};

	s = strrchr(name,'/');
	s1 = strrchr(name,'.');

	if(!s1){
		gchar *msg = g_strconcat("File ",name," is not a Titano project!",NULL);
		ask_error(msg);
		g_free(msg);
		g_free(name);
		return;
	};

	if(g_strcasecmp(s1+1,"titano")){
		gchar *msg = g_strconcat("File ",name," is not a Titano project!",NULL);
		ask_error(msg);
		g_free(msg);
		g_free(name);
		return;
	};
	
	fname = g_strndup(s+1,s1-s-1);
	dir = g_strndup(name,s-name);

	read_conf(name,"/General/type",type);
	read_conf(name,"/General/rev",rev);
	
	delete cur_project;
	cur_project = NULL;

	gboolean revnr = FALSE;
	if(!g_strcasecmp(type,"cpp")){
		if(revnr = !CCppProject::can_translate_format(rev))
			ask_error("Couldn't recognize format of C/C++ project.");
		else
			cur_project = new CCppProject(fname,dir,mainapp,mvpaned,c_list,o_list,e_list);
	}

	if(!cur_project&&!revnr)
		ask_error("Couldn't recognize type of project.");

	g_free(name);
	g_free(fname);
	g_free(dir);
	
	up_interface();
}

void
proj_saveas_ok(GtkWidget *widget,GtkFileSelection *fs)
{
	gchar *name, *msg, *s;
	gint ans;
	
	name = g_strdup(gtk_file_selection_get_filename(fs));
	gtk_widget_destroy(GTK_WIDGET(fs));

	s = strrchr(name,'.');
	if(!s){
		s = g_strconcat(name,".titano",NULL);
		name = s;
	}else{
		if(!g_strcasecmp("titano",s+1)){
			s = g_strconcat(name,".titano",NULL);
			g_free(name);
			name = s;
		};
	};
	
	if(file_exists(name)){
		msg = g_strconcat("File ",name," already exists. Overwrite?",NULL);
		ans = ask_yes_no(msg);
		g_free(msg);

		if(ans){
			g_free(name);
			return;
		};
	};

	cur_project->save_as(name);
	
	g_free(name);
}

void
file_saveas_ok(GtkWidget *widget,GtkFileSelection *fs)
{
	gchar *name, *msg;
	gint ans;
	
	name = g_strdup(gtk_file_selection_get_filename(fs));
	gtk_widget_destroy(GTK_WIDGET(fs));

	if(file_exists(name)){
		msg = g_strconcat("File ",name," already exists. Overwrite?",NULL);
		ans = ask_yes_no(msg);
		g_free(msg);

		if(ans){
			g_free(name);
			return;
		};
	};

	cur_project->save_file_as(name);
	
	g_free(name);
}

void
add_file_ok(GtkWidget *widget,GtkFileSelection *fs)
{
	gchar *name, *msg;
	gint ans;
	
	name = g_strdup(gtk_file_selection_get_filename(fs));
	gtk_widget_destroy(GTK_WIDGET(fs));

	if(!file_exists(name)){
		msg = g_strconcat("File ",name," does not exists. Create?",NULL);
		ans = ask_yes_no(msg);
		g_free(msg);

		if(!ans){
			file_create(name);
		}else{
			g_free(name);
			return;
		};
	};

	cur_project->add_file(name);
	
	g_free(name);
	
	up_interface();
}




