#include <sys/types.h>
#include <sys/wait.h>

#include "projects.h"
#include "dialogs.h"
#include "util.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

//XPM's
#include "xpm/compile.xpm"
#include "xpm/debug.xpm"
#include "xpm/exec.xpm"

//declarations of dynamic menus used by CCppProject
static GnomeUIInfo comp_menu_uiinfo[] =
{
	{ GNOME_APP_UI_ITEM, N_("_Make"), N_("Make current project"), on_make_activate, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_EXEC, GDK_F9, GDK_CONTROL_MASK, 0 },
	{ GNOME_APP_UI_ITEM, N_("_Compile"), N_("Compile current file"), on_compile_activate, 0, 0, GNOME_APP_PIXMAP_DATA, compile_xpm, GDK_F9, GDK_MOD1_MASK, 0 },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Debug"), N_("Debug current project"), on_debug_activate, 0, 0, GNOME_APP_PIXMAP_DATA, debug_xpm, GDK_F5, (GdkModifierType) 0, 0 },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Execute"), N_("Execute current project"), on_execute_activate, 0, 0, GNOME_APP_PIXMAP_DATA, exec_xpm, GDK_F9, (GdkModifierType) 0, 0 },
	GNOMEUIINFO_END
};

static GnomeUIInfo cpp_menu_uiinfo[] =
{
	{ GNOME_APP_UI_ITEM, N_("Comment _selection"), N_("Comment currently selected region"), on_comsel_activate, 0, 0, GNOME_APP_PIXMAP_NONE, NULL, 0, (GdkModifierType) 0, 0 },
	{ GNOME_APP_UI_ITEM, N_("Comment _line"), N_("Comment current line"), on_comline_activate, 0, 0, GNOME_APP_PIXMAP_NONE, NULL, 0, (GdkModifierType) 0, 0 },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Backward statement"), N_("Move cursor to the previous statement"), on_back_activate, 0, 0, GNOME_APP_PIXMAP_NONE, NULL, 0, (GdkModifierType) 0, 0 },
	{ GNOME_APP_UI_ITEM, N_("_Forward statement"), N_("Move cursor to the previous statement"), on_forw_activate, 0, 0, GNOME_APP_PIXMAP_NONE, NULL, 0, (GdkModifierType) 0, 0 },
	GNOMEUIINFO_END
};

static GnomeUIInfo compmenubar_uiinfo[] =
{
	GNOMEUIINFO_SUBTREE(N_("_Compile"),comp_menu_uiinfo),
	GNOMEUIINFO_SUBTREE(N_("C/C_++"),cpp_menu_uiinfo),
	GNOMEUIINFO_END
};

//input callbacks
void make_cb(gint *pid,gint file,GdkInputCondition cond)
{
	gchar output[384] = "/0";
	gchar *line[1];
	gint stat, code;

	if(!fgets(output,121,cur_project->make_f)){
		gdk_input_remove(cur_project->make_id);
		cur_project->make_id = 0;
		code = pclose(cur_project->make_f);
		g_snprintf(output,384,"Make exit status: %d",code);
		line[0] = output;
		gtk_clist_append(GTK_CLIST(cur_project->compile_list),line);
		gtk_clist_columns_autosize(GTK_CLIST(cur_project->compile_list));
	}else{
		if(gchar *pos = strchr(output,'\n'))
			*pos = '\0';
		line[0] = output;
		gtk_clist_append(GTK_CLIST(cur_project->compile_list),line);
	};
}

void compile_cb(gint *pid,gint file,GdkInputCondition cond)
{
	gchar output[384] = "/0";
	gchar *line[1];
	gint stat, code;

	if(!fgets(output,121,cur_project->comp_f)){
		gdk_input_remove(cur_project->comp_id);
		cur_project->comp_id = 0;
		code = pclose(cur_project->comp_f);
		g_snprintf(output,384,"Compile exit status: %d",code);
		line[0] = output;
		gtk_clist_append(GTK_CLIST(cur_project->compile_list),line);
		gtk_clist_columns_autosize(GTK_CLIST(cur_project->compile_list));		
	}else{
		if(gchar *pos = strchr(output,'\n'))
			*pos = '\0';
		line[0] = output;
		gtk_clist_append(GTK_CLIST(cur_project->compile_list),line);
	};
}

gint debug_cb(gpointer pid)
{
	gchar out[80];
	gchar npid[5];
	FILE *f;

	g_snprintf(npid,sizeof(npid),"%d",(gint)pid);
	gchar *name = g_strconcat("/proc/",npid,"/stat",NULL);
	f = fopen(name,"r");
	fgets(out,sizeof(out),f);
	fclose(f);
	g_free(name);
	if(strchr(out,'Z')){
		waitpid((gint)pid,NULL,0);
		return 0;
	};

	return 1;
}

void exec_out_cb(gint *pid,gint file,GdkInputCondition cond)
{
	gchar output[384] = "/0";
	gchar *line[1];
	gint stat, code;
	
	if(!read(file,output,sizeof(output))){
		gdk_input_remove(cur_project->exec_out_id);
		cur_project->exec_out_id = 0;
		waitpid((gint)pid,&stat,0);
		if(WIFEXITED(stat))
			code = WEXITSTATUS(stat);
		else
			code = -1;
		g_snprintf(output,384,"Program exit status: %d",code);
		line[0] = output;
		gtk_clist_append(GTK_CLIST(cur_project->stdout_list),line);
	}else{
		gchar *pos = output, *start = output;
		while(gchar *pos = strchr(start,'\n')){
			*pos = '\0';
			line[0] = start;
			gtk_clist_append(GTK_CLIST(cur_project->stdout_list),line);
			start = pos+1;
		};	
	};
}

void exec_err_cb(gint *pid,gint file,GdkInputCondition cond)
{
	gchar output[384] = "/0";
	gchar *line[1];

	if(!read(file,output,sizeof(output))){
		gdk_input_remove(cur_project->exec_err_id);
		cur_project->exec_err_id = 0;
	}else{
		gchar *pos = output, *start = output;
		while(gchar *pos = strchr(start,'\n')){
			*pos = '\0';
			line[0] = start;
			gtk_clist_append(GTK_CLIST(cur_project->stderr_list),line);
			start = pos+1;
		};	
	};
}

//CCppProject class
CCppProject::CCppProject(gchar *pname,gchar *d,GtkWidget *a,GtkWidget *vpaned,GtkWidget *clist,GtkWidget *olist,GtkWidget *elist)
{
	GtkWidget *wind;

	cfiles = hfiles = NULL;
	cfiles_cnt = hfiles_cnt = 0;

	open_cfiles = open_hfiles = NULL;
	open_c_cnt = open_h_cnt = 0;

	name = g_strdup(pname);
	dir = g_strdup(d);
	wdir = g_strdup(d);

	cmake = comp = deb = exec = NULL;
	make_id = comp_id = exec_out_id = exec_err_id = 0;

	hilite = NULL;
	
	compile_list = clist;
	stdout_list = olist;
	stderr_list = elist;
	app = a;

	cur_file = NULL;
				
	//creating widgets
	window = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(window),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_show(window);
	hpaned1 = gtk_hpaned_new();
	gtk_paned_set_gutter_size(GTK_PANED(hpaned1),12);
	gtk_paned_set_position(GTK_PANED(hpaned1),120);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(window),hpaned1);
	gtk_widget_show(hpaned1);
	
	wind = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(wind),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_show(wind);
	projtree = gtk_tree_new();   //for proj explorer files
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(wind),projtree);
	gtk_widget_show(projtree);
	
	hitem = gtk_tree_item_new_with_label("Headers");
	gtk_widget_show(hitem);
	gtk_tree_append(GTK_TREE(projtree),hitem);
	htree = gtk_tree_new();
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(hitem),htree);
	
	citem = gtk_tree_item_new_with_label("Sources");
	gtk_widget_show(citem);
	gtk_tree_append(GTK_TREE(projtree),citem);
	ctree = gtk_tree_new();
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(citem),ctree);

	oitem = gtk_tree_item_new_with_label("Other");
	gtk_widget_show(oitem);
	gtk_tree_append(GTK_TREE(projtree),oitem);
	otree = gtk_tree_new();
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(oitem),otree);

	gtk_paned_add1(GTK_PANED(hpaned1),GTK_WIDGET(wind));

	wind = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(wind),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_show(wind);
	hpaned2 = gtk_hpaned_new();   //for proj c&h files
	gtk_paned_set_gutter_size(GTK_PANED(hpaned2),12);
	gtk_paned_set_position(GTK_PANED(hpaned2),300);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(wind),hpaned2);
	gtk_widget_show(hpaned2);

	gtk_paned_add2(GTK_PANED(hpaned1),GTK_WIDGET(wind));

	hnotebook = gtk_notebook_new();
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(hnotebook),TRUE);
	gtk_signal_connect(GTK_OBJECT(hnotebook),"switch_page",GTK_SIGNAL_FUNC(files_chg_cb),NULL);
	gtk_signal_connect(GTK_OBJECT(hnotebook),"set_focus_child",GTK_SIGNAL_FUNC(notebook_child_focus_cb),NULL);
	gtk_signal_connect(GTK_OBJECT(hnotebook),"focus",GTK_SIGNAL_FUNC(notebook_focus_cb),NULL);
	gtk_paned_add1(GTK_PANED(hpaned2),GTK_WIDGET(hnotebook));
	gtk_widget_show(hnotebook);
	
	cnotebook = gtk_notebook_new();
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(cnotebook),TRUE);
	gtk_signal_connect(GTK_OBJECT(cnotebook),"switch_page",GTK_SIGNAL_FUNC(files_chg_cb),NULL);
	gtk_signal_connect(GTK_OBJECT(cnotebook),"set_focus_child",GTK_SIGNAL_FUNC(notebook_child_focus_cb),NULL);
	gtk_signal_connect(GTK_OBJECT(cnotebook),"focus",GTK_SIGNAL_FUNC(notebook_focus_cb),NULL);
	gtk_paned_add2(GTK_PANED(hpaned2),GTK_WIDGET(cnotebook));
	gtk_widget_show(cnotebook);

	gtk_paned_add1(GTK_PANED(vpaned),window);

	create_menus();

	while(gtk_events_pending())
		gtk_main_iteration();
	
	load_highlights();
	load();
	set_title();
}

CCppProject::~CCppProject()
{
	gint i;
	CFile *f;

	destroy_menus();
	
	g_free(name);
	g_free(dir);
	g_free(wdir);
	if(hilite)
		g_free(hilite);

	for(i = open_c_cnt-1;i >= 0;i--){
		f = (CFile *)g_slist_nth_data(open_cfiles,i);
		delete f;
	};

	for(i = open_h_cnt-1;i >= 0;i--){
		f = (CFile *)g_slist_nth_data(open_hfiles,i);
		delete f;
	};

	g_slist_free(open_cfiles);
	g_slist_free(open_hfiles);

	for(i = cfiles_cnt-1;i >= 0;i--){
		g_free((gchar *)g_slist_nth_data(cfiles,i));
	};

	for(i = hfiles_cnt-1;i >= 0;i--){
		g_free((gchar *)g_slist_nth_data(hfiles,i));
	};

	g_slist_free(cfiles);
	g_slist_free(hfiles);
	
	gtk_widget_destroy(window);
}

gint CCppProject::create_empty(gchar *name,gchar *dir,gchar *def_make,gchar *def_compile,gchar *def_debug,gchar *def_exec)
{
	gchar *nname, *msg;
	gint ans;

	nname = g_strconcat(dir,"/",name,".titano",NULL);

	if(file_exists(nname)){
		msg = g_strconcat("File ",nname," already exists. Overwrite?",NULL);
		ans = ask_yes_no(msg);
		g_free(msg);
		if(ans){
			g_free(nname);
			return 1;
		};
	};

	write_conf(nname,"/General/type","cpp");
	write_conf(nname,"/General/rev",1);
	write_conf(nname,"Project/name",name);
	write_conf(nname,"Project/wdir",dir);
	write_conf(nname,"Commands/make",def_make);
	write_conf(nname,"Commands/compile",def_compile);
	write_conf(nname,"Commands/debug",def_debug);
	write_conf(nname,"Commands/execute",def_exec);

	clear_sect_conf(nname,"Files");
	write_conf(nname,"Files/empty","empty");
	clear_key_conf(nname,"Files/empty");
	
	drop_conf(nname);
	
	g_free(nname);

	return 0;
}

void CCppProject::set_title(void)
{
	gchar buf[40] = "Titano " VERSION  " - ";
	gchar *title;

	if(name)
		title = g_strconcat(buf,name,NULL);
	gtk_window_set_title(GTK_WINDOW(app),title);
	g_free(title);
}

void CCppProject::load(void)
{
	gchar *nname, *ch, buf[378];
	gchar *fn,*open;
	GtkWidget *item;
	void *iterator;

	while(gtk_events_pending())
		gtk_main_iteration();

	nname = g_strconcat(dir,"/",name,".titano",NULL);

	g_free(wdir);
	read_conf(nname,"/Project/wdir",wdir);
	chdir(wdir);

	read_conf(nname,"Commands/make",cmake);
	read_conf(nname,"Commands/compile",comp);
	read_conf(nname,"Commands/debug",deb);
	read_conf(nname,"Commands/execute",exec);

	ch = g_strconcat("=",nname,"=/Files",NULL);
	iterator = gnome_config_init_iterator(ch);
	g_free(ch);
	
	iterator = gnome_config_iterator_next(iterator,&fn,&open);
	while(iterator){                                   //reading files
		if(!file_exists(fn)){
			gchar *msg;
			gint ans;

			msg = g_strconcat("File ",fn," doesn't exists. Create new?",NULL);
			ans = ask_yes_no(msg);
			g_free(msg);
			
			if(!ans)
				file_create(fn);
			else{
				iterator = gnome_config_iterator_next(iterator,&fn,&open);
				continue;				
			};
		};
		item = gtk_tree_item_new_with_label(fn);
		gtk_signal_connect(GTK_OBJECT(item),"select",GTK_SIGNAL_FUNC(item_selected),g_strdup(fn));
		gtk_signal_connect(GTK_OBJECT(item),"deselect",GTK_SIGNAL_FUNC(item_selected),g_strdup(fn));
		gtk_widget_show(item);
		if(!g_strcasecmp(strrchr(fn,'.'),".h")){
			hfiles_cnt++;
			hfiles = g_slist_append(hfiles,g_strdup(fn));
			gtk_tree_append(GTK_TREE(htree),item);
		}else{
			cfiles_cnt++;
			cfiles = g_slist_append(cfiles,g_strdup(fn));
			if(!g_strcasecmp(strrchr(fn,'.'),".c")||
			   !g_strcasecmp(strrchr(fn,'.'),".C")||
			   !g_strcasecmp(strrchr(fn,'.'),".cc")||
			   !g_strcasecmp(strrchr(fn,'.'),".cpp")||
			   !g_strcasecmp(strrchr(fn,'.'),".cxx"))
				gtk_tree_append(GTK_TREE(ctree),item);
			else
				gtk_tree_append(GTK_TREE(otree),item);
		};
		
		if((open[0] == '1')&&file_exists(fn))
			open_file(fn);
		
		iterator = gnome_config_iterator_next(iterator,&fn,&open);	
       	};

	g_free(nname);
}

void CCppProject::load(GtkWidget *propbox)
{
	GtkWidget *entry;
	gchar *newv, *oldname, *newname, *fname;
	GSList *newcfiles = NULL, *newhfiles = NULL;

	entry = lookup_widget(propbox,"projname");
	newv = gtk_entry_get_text(GTK_ENTRY(entry));
	if(newv){
		g_free(name);
		name = g_strdup(newv);
	};

	entry = lookup_widget(propbox,"projwdir");
	newv = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(entry),TRUE);
	if(newv){
		gchar *oldwdir = wdir;
		wdir = g_strdup(newv);
		int i;
		//tuning all file names
		for(i = 0;i <= cfiles_cnt-1;i++){
			oldname = (gchar *)g_slist_nth_data(cfiles,i);
			fname = fullname(oldwdir,oldname);
			newname = form_name(fname);
			g_free(fname);
			newcfiles = g_slist_append(newcfiles,newname);

			if(file_is_opened(oldname))
				get_c_file(oldname)->set_name(newname);
		};

		for(i = 0;i <= hfiles_cnt-1;i++){
			oldname = (gchar *)g_slist_nth_data(hfiles,i);
			fname = fullname(oldwdir,oldname);
			newname = form_name(fname);
			g_free(fname);
			newhfiles = g_slist_append(newhfiles,newname);

			if(file_is_opened(oldname))
				get_h_file(oldname)->set_name(newname);
		};
		//setting new lists
		for(i = cfiles_cnt-1;i >= 0;i--){
			g_free((gchar *)g_slist_nth_data(cfiles,i));
		};
		
		for(i = hfiles_cnt-1;i >= 0;i--){
			g_free((gchar *)g_slist_nth_data(hfiles,i));
		};
		g_slist_free(cfiles);
		g_slist_free(hfiles);
		cfiles = newcfiles;
		hfiles = newhfiles;
		
		g_free(oldwdir);
	};

	entry = lookup_widget(propbox,"projmake");
	newv = gtk_entry_get_text(GTK_ENTRY(entry));
	if(newv){
		g_free(cmake);
		cmake = g_strdup(newv);
	};

	entry = lookup_widget(propbox,"projcomp");
	newv = gtk_entry_get_text(GTK_ENTRY(entry));
	if(newv){
		g_free(comp);
		comp = g_strdup(newv);
	};

	entry = lookup_widget(propbox,"projdebug");
	newv = gtk_entry_get_text(GTK_ENTRY(entry));
	if(newv){
		g_free(deb);
		deb = g_strdup(newv);
	};

	entry = lookup_widget(propbox,"projexec");
	newv = gtk_entry_get_text(GTK_ENTRY(entry));
	if(newv){
		g_free(exec);
		exec = g_strdup(newv);
	};

	chdir(wdir);
	set_title();
}

void CCppProject::save(void)
{
	FILE *f;
	gint i;
	gchar *nname = g_strconcat(dir,"/",name,".titano",NULL);
	gchar *namef;
	CFile *fn;
	
	f = fopen(nname,"w");
	if(!f){
		ask_error("Save project failed.");
		return;
	};
	fclose(f);
	write_conf(nname,"/General/type","cpp");
	write_conf(nname,"/General/rev",1);
	write_conf(nname,"Project/name",name);
	write_conf(nname,"Project/wdir",wdir);
	write_conf(nname,"Commands/make",cmake);
	write_conf(nname,"Commands/compile",comp);
	write_conf(nname,"Commands/debug",deb);
	write_conf(nname,"Commands/execute",exec);

	clear_sect_conf(nname,"Files");
	write_conf(nname,"Files/empty","empty");
	clear_key_conf(nname,"Files/empty");
	
	for(i = 0;i <= hfiles_cnt-1;i++){
		namef = g_strconcat("/Files/",(gchar *)g_slist_nth_data(hfiles,i),NULL);
		write_conf(nname,namef,file_is_opened((gchar *)g_slist_nth_data(hfiles,i)));
		g_free(namef);
	};
	for(i = 0;i <= cfiles_cnt-1;i++){
		namef = g_strconcat("/Files/",(gchar *)g_slist_nth_data(cfiles,i),NULL);
		write_conf(nname,namef,file_is_opened((gchar *)g_slist_nth_data(cfiles,i)));
		g_free(namef);
	};
	
	drop_conf(nname);

	g_free(nname);
}

void CCppProject::save(GtkWidget *propbox)
{
	GtkWidget *entry;

	entry = lookup_widget(propbox,"projname");
	gtk_entry_set_text(GTK_ENTRY(entry),name);
	
	entry = lookup_widget(propbox,"projwdir");
	gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(entry),wdir);
	gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(entry))),wdir);
	
	entry = lookup_widget(propbox,"projmake");
	gtk_entry_set_text(GTK_ENTRY(entry),cmake);

	entry = lookup_widget(propbox,"projcomp");
	gtk_entry_set_text(GTK_ENTRY(entry),comp);

	entry = lookup_widget(propbox,"projdebug");
	gtk_entry_set_text(GTK_ENTRY(entry),deb);

	entry = lookup_widget(propbox,"projexec");
	gtk_entry_set_text(GTK_ENTRY(entry),exec);
}

void CCppProject::save_as(gchar *newname)
{
	gchar *s,*s1;

	g_free(name);
	g_free(dir);

	s = strrchr(newname,'/')+1;
	s1 = strrchr(newname,'.');

	name = g_strndup(s,s1-s);
	dir = g_strndup(newname,s-newname);

	save();
	set_title();
}

gboolean CCppProject::can_translate_format(gint form)
{
	return form = 1;
}

void CCppProject::new_file(gchar *name)
{
	GtkWidget *item;
	
	file_create(name);

	item = gtk_tree_item_new_with_label(name);
	gtk_signal_connect(GTK_OBJECT(item),"select",GTK_SIGNAL_FUNC(item_selected),name);
	gtk_signal_connect(GTK_OBJECT(item),"deselect",GTK_SIGNAL_FUNC(item_selected),name);
	gtk_widget_show(item);
	if(name[strlen(name)-1] == 'h'){
		hfiles = g_slist_append(hfiles,g_strdup(name));
		hfiles_cnt++;
		gtk_tree_append(GTK_TREE(htree),item);
		gtk_tree_item_expand(GTK_TREE_ITEM(hitem));
	}else{
       		cfiles = g_slist_append(cfiles,g_strdup(name));
		cfiles_cnt++;
		gtk_tree_append(GTK_TREE(ctree),item);
		gtk_tree_item_expand(GTK_TREE_ITEM(citem));
	};

	open_file(name);
}

void CCppProject::open_file(gchar *name)
{
	CFile *nfile = NULL;

	if(!g_strcasecmp(strrchr(name,'.'),".h")){
		nfile = new CHFile(name,hnotebook,hilite);
		open_hfiles = g_slist_append(open_hfiles,nfile);
		open_h_cnt++;
	}else{
		if(!g_strcasecmp(strrchr(name,'.'),".c")||
		   !g_strcasecmp(strrchr(name,'.'),".C")||
		   !g_strcasecmp(strrchr(name,'.'),".cc")||
		   !g_strcasecmp(strrchr(name,'.'),".cpp")||
		   !g_strcasecmp(strrchr(name,'.'),".cxx"))
			nfile = new CCFile(name,cnotebook,hilite);
		else
			nfile = new CFile(name,cnotebook,hilite);
		open_cfiles = g_slist_append(open_cfiles,nfile);
		open_c_cnt++;
	};

	cur_file = nfile;
	cur_file->activate();
}

void CCppProject::save_file_as(gchar *newname)
{
	GSList *list, *el;
	gchar *fname = cur_file->get_name();
	gchar *nname = form_name(newname);

	if(fname[strlen(fname)-1] == 'h')
		list = hfiles;
	else
		list = cfiles;
	el = g_slist_find_custom(list,fname,strcomp);

	if(!cur_file->save_as(nname)){
		g_free((gchar *)(el->data));
		el->data = (void *)g_strdup(nname);
	};

	g_free(nname);
}

void CCppProject::close_file(void)
{
	gchar *name;
	CFile *file;

	name = cur_file->get_name();
	if(cur_file->modified()){
		gchar *msg = g_strconcat("File ",cur_file->get_name()," was modified. Save?",NULL);
		gint ans = ask_yes_no_cancel(msg);
		g_free(msg);
		
		switch(ans){
		case 0:cur_file->save();
		case 2:return;
		};
	};

	file = cur_file;
	cur_file = NULL;
	
	if(!g_strcasecmp(strrchr(name,'.'),".h")){
		open_hfiles = g_slist_remove(open_hfiles,file);
		open_h_cnt--;
	}else{
		open_cfiles = g_slist_remove(open_cfiles,file);
		open_c_cnt--;
	};
	
	if(!g_strcasecmp(strrchr(name,'.'),".h")){
		if(!open_h_cnt&&open_c_cnt){
			gint n = gtk_notebook_current_page(GTK_NOTEBOOK(cnotebook));
			GtkWidget *p = gtk_notebook_get_nth_page(GTK_NOTEBOOK(cnotebook),n);
			GtkWidget *l = gtk_notebook_get_tab_label(GTK_NOTEBOOK(cnotebook),p);
			gtk_label_get(GTK_LABEL(l),&name);

			get_c_file(name)->activate();
		};
	}else{
		if(!open_c_cnt&&open_h_cnt){
			gint n = gtk_notebook_current_page(GTK_NOTEBOOK(hnotebook));
			GtkWidget *p = gtk_notebook_get_nth_page(GTK_NOTEBOOK(hnotebook),n);
			GtkWidget *l = gtk_notebook_get_tab_label(GTK_NOTEBOOK(hnotebook),p);
			gtk_label_get(GTK_LABEL(l),&name);

			get_h_file(name)->activate();
		};
	};

	delete file;
}

void CCppProject::close_file(char *name)
{
	CFile *file;

	if(!g_strcasecmp(name,cur_file->get_name())){
		close_file();
		return;
	};

	if(!g_strcasecmp(strrchr(name,'.'),".h"))
		file = get_h_file(name);
	else
		file = get_c_file(name);
		
	if(file->modified()){
		gchar *msg = g_strconcat("File ",file->get_name()," was modified. Save?",NULL);
		gint ans = ask_yes_no_cancel(msg);
		g_free(msg);
		
		switch(ans){
		case 0:file->save();
		case 2:return;
		};
	};
	
	if(!g_strcasecmp(strrchr(name,'.'),".h")){
		open_hfiles = g_slist_remove(open_hfiles,file);
		open_h_cnt--;
	}else{
		open_cfiles = g_slist_remove(open_cfiles,file);
		open_c_cnt--;
	};

	delete file;
}

void CCppProject::save_all_files(void)
{
	gint i;
	CFile *file;

	for(i = open_c_cnt-1;i >= 0;i--){
		file = (CFile *)g_slist_nth_data(open_cfiles,i);
		if(file->modified())
			file->save();
	};

	for(i = open_h_cnt-1;i >= 0;i--){
		file = (CFile *)g_slist_nth_data(open_hfiles,i);
		if(file->modified())
			file->save();
	};
}

gboolean CCppProject::has_modif_files(void)
{
	gint i;
	gboolean has = FALSE;

	for(i = open_h_cnt-1;(i >= 0)&&!has;i--)
		has = has||((CFile *)g_slist_nth_data(open_hfiles,i))->modified();

	for(i = open_c_cnt-1;(i >= 0)&&!has;i--)
		has = has||((CFile *)g_slist_nth_data(open_cfiles,i))->modified();
}

void CCppProject::fill_file_list(GtkList *list)
{
	GList *l = NULL;
	gint i;
	GtkWidget *list_item;

	for(i = 0;i <= hfiles_cnt-1;i++){
		GtkWidget *label;
             
		label = gtk_label_new((gchar *)g_slist_nth_data(hfiles,i));
		list_item = gtk_list_item_new();
		gtk_container_add(GTK_CONTAINER(list_item),label);
		gtk_widget_show(label);
		gtk_widget_show(list_item);

		l = g_list_append(l,list_item);
	};

	for(i = 0;i <= cfiles_cnt-1;i++){
		GtkWidget *label;
             
		label = gtk_label_new((gchar *)g_slist_nth_data(cfiles,i));
		list_item = gtk_list_item_new();
		gtk_container_add(GTK_CONTAINER(list_item),label);
		gtk_widget_show(label);
		gtk_widget_show(list_item);

		l = g_list_append(l,list_item);
	};

	gtk_list_append_items(list,l);
}

void CCppProject::activate_file(gchar *name)
{
	CFile *file;
	
	if(!file_is_opened(name))
		return;

	if(!g_strcasecmp(strrchr(name,'.'),".h"))
		file = get_h_file(name);
	else
		file = get_c_file(name);

	file->activate();
}

void CCppProject::add_file(gchar *name)
{
	GtkWidget *item;
	gchar *nname = form_name(name);

	if(file_in_project(nname)){
		gchar *msg;

		msg = g_strconcat("File ",name," it wouldn't be added.",NULL);
		ask_error(msg);
		g_free(msg);
	};

	item = gtk_tree_item_new_with_label(nname);
	gtk_signal_connect(GTK_OBJECT(item),"select",GTK_SIGNAL_FUNC(item_selected),nname);
	gtk_signal_connect(GTK_OBJECT(item),"deselect",GTK_SIGNAL_FUNC(item_selected),nname);
	gtk_widget_show(item);
	if(!g_strcasecmp(strrchr(nname,'.'),".h")){
		hfiles = g_slist_append(hfiles,nname);
		hfiles_cnt++;
		gtk_tree_append(GTK_TREE(htree),item);
		gtk_tree_item_expand(GTK_TREE_ITEM(hitem));
	}else{
		cfiles = g_slist_append(cfiles,nname);
		cfiles_cnt++;
		if(!g_strcasecmp(strrchr(name,'.'),".c")||
		   !g_strcasecmp(strrchr(name,'.'),".C")||
		   !g_strcasecmp(strrchr(name,'.'),".cc")||
		   !g_strcasecmp(strrchr(name,'.'),".cpp")||
		   !g_strcasecmp(strrchr(name,'.'),".cxx")){
			gtk_tree_append(GTK_TREE(ctree),item);
			gtk_tree_item_expand(GTK_TREE_ITEM(citem));
		}else{
			gtk_tree_append(GTK_TREE(otree),item);
			gtk_tree_item_expand(GTK_TREE_ITEM(oitem));
		};
	};

	open_file(nname);
	
	save();
}

void CCppProject::remove_file(gchar *name)
{
	GSList *el;
	GList *treeitem;
	gint i;

	if(file_is_opened(name))
		close_file(name);

	if(!g_strcasecmp(strrchr(name,'.'),".h")){
		el = g_slist_find_custom(hfiles,name,strcomp);
		i = g_slist_index(hfiles,el->data);
		treeitem = g_list_nth(GTK_TREE(htree)->children,i);
		gtk_container_remove(GTK_CONTAINER(htree),GTK_WIDGET(treeitem->data));
		gtk_tree_item_expand(GTK_TREE_ITEM(hitem));
		hfiles = g_slist_remove(hfiles,el->data);
		hfiles_cnt--;
	}else{
		if(!g_strcasecmp(strrchr(name,'.'),".c")||
		   !g_strcasecmp(strrchr(name,'.'),".C")||
		   !g_strcasecmp(strrchr(name,'.'),".cc")||
		   !g_strcasecmp(strrchr(name,'.'),".cpp")||
		   !g_strcasecmp(strrchr(name,'.'),".cxx")){
			treeitem = GTK_TREE(ctree)->children;
			while(g_strcasecmp(name,((GtkLabel *)(((GtkBin *)(treeitem->data))->child))->label))
				treeitem = treeitem->next;

			gtk_widget_ref(ctree);
			gtk_container_remove(GTK_CONTAINER(ctree),GTK_WIDGET(treeitem->data));
			if(ctree->parent == NULL)
				gtk_tree_item_set_subtree(GTK_TREE_ITEM(citem),ctree);
			else
				gtk_widget_unref(ctree);
			gtk_tree_item_expand(GTK_TREE_ITEM(citem));
		}else{
			treeitem = GTK_TREE(otree)->children;
			while(g_strcasecmp(name,((GtkLabel *)(((GtkBin *)(treeitem->data))->child))->label))
				treeitem = treeitem->next;
			gtk_widget_ref(otree);
			gtk_container_remove(GTK_CONTAINER(otree),GTK_WIDGET(treeitem->data));
			if(otree->parent == NULL)
				gtk_tree_item_set_subtree(GTK_TREE_ITEM(oitem),otree);
			else
				gtk_widget_unref(otree);
			gtk_tree_item_expand(GTK_TREE_ITEM(oitem));
		};

		el = g_slist_find_custom(cfiles,name,strcomp);
		cfiles = g_slist_remove(cfiles,el->data);
		cfiles_cnt--;
	};
	save();
}

void CCppProject::load_highlights(void)
{
	gchar *path = g_strconcat(libdir,"/c-scanner",NULL);

	if(!hilite){
		hilite = (HighlightSet *)malloc(sizeof(HighlightSet));
		
		hilite->scanner = gtk_editor_load_scanner(path);
	};		

	g_free(path);
}

gboolean CCppProject::file_in_project(gchar *name)
{
	gboolean a, b;
	a = g_slist_find_custom(hfiles,name,strcomp) != NULL;
	b = g_slist_find_custom(cfiles,name,strcomp) != NULL;
	return a||b;
}

gboolean CCppProject::file_is_opened(gchar *name)
{
	gboolean a, b;
	a = g_slist_find_custom(open_hfiles,name,filescmp) != NULL;
	b = g_slist_find_custom(open_cfiles,name,filescmp) != NULL;
	return a||b;
}


CHFile *CCppProject::get_h_file(gchar *name)
{
	GSList *el;

	el = g_slist_find_custom(open_hfiles,name,filescmp);
	if(el)
		return (CHFile *)(el->data);
	else
		return NULL;
}

CCFile *CCppProject::get_c_file(gchar *name)
{
	GSList *el;

	el = g_slist_find_custom(open_cfiles,name,filescmp);
	if(el)
		return (CCFile *)(el->data);
	else
		return NULL;
}

void CCppProject::set_file(gchar *name)
{
	CFile *newcur;	

	if(!g_strcasecmp(strrchr(name,'.'),".h"))
		newcur = get_h_file(name);
	else
		newcur = get_c_file(name);
	
	if(newcur)
		cur_file = newcur;
}

void CCppProject::create_menus(void)
{
	gnome_app_insert_menus(GNOME_APP(mainapp),"_Project",compmenubar_uiinfo);
	gnome_app_install_menu_hints(GNOME_APP(mainapp),compmenubar_uiinfo);
}

void CCppProject::destroy_menus(void)
{
	gnome_app_remove_menus(GNOME_APP(mainapp),"_Compile",2);
}


void CCppProject::make(void)
{
	gchar *command = prepare_command(cmake);
	gchar *commandi = g_strconcat(command," 2>&1",NULL);
	gchar *line[1];

	if(!make_id){
		make_f = popen(commandi,"r");
		if(!make_f){
			g_print("Failed to create child process.");
			return;
		};
		line[0] = command;
		gtk_clist_clear(GTK_CLIST(compile_list));
		gtk_clist_append(GTK_CLIST(compile_list),line);
		make_id = gdk_input_add(fileno(make_f),GDK_INPUT_READ,make_cb,NULL);
	};

	g_free(command);
	g_free(commandi);
}

void CCppProject::compile(void)
{
	gchar *command = prepare_command(comp);
	gchar *commandi = g_strconcat(command," 2>&1",NULL);
	gchar *line[1];

	if(!comp_id){
		comp_f = popen(commandi,"r");
		if(!comp_f){
			g_print("Failed to create child process.");
			return;
		};
		line[0] = command;
		gtk_clist_clear(GTK_CLIST(compile_list));
		gtk_clist_append(GTK_CLIST(compile_list),line);
		comp_id = gdk_input_add(fileno(comp_f),GDK_INPUT_READ,compile_cb,NULL);
	};

	g_free(command);
	g_free(commandi);
}

void CCppProject::debug(void)
{
	gchar *command = prepare_command(deb);
	gchar **args = g_strsplit(command," ",127);
	gint pid;
	
	g_free(command);
	if((pid = fork()) == -1){
		g_strfreev(args);
		return;
	};
	if(pid == 0){
		//child
		execvp(args[0],args);
		_exit(1);
	}else{
		//parent
		gtk_timeout_add(1000,debug_cb,(void *)pid);
	};
	g_strfreev(args);
}

void CCppProject::execute(void)
{
	gchar *command = prepare_command(exec);
	gchar **args = g_strsplit(command," ",127);
	gint epipe_o[2], epipe_e[2];
	gint pid;
	gchar *line[1];

	if(!comp_id){
		pipe(epipe_o);
		pipe(epipe_e);
		if((pid = fork()) == -1){
			g_print("Failed to create child process.");
			g_strfreev(args);
			return;
		};
		if(pid != 0){
			//parent
			close(epipe_o[1]);
			close(epipe_e[1]);
			line[0] = command;
			gtk_clist_clear(GTK_CLIST(stdout_list));
			gtk_clist_append(GTK_CLIST(stdout_list),line);
			gtk_clist_clear(GTK_CLIST(stderr_list));
			exec_out_id = gdk_input_add(epipe_o[0],GDK_INPUT_READ,exec_out_cb,(void *)pid);
			exec_err_id = gdk_input_add(epipe_e[0],GDK_INPUT_READ,exec_err_cb,(void *)pid);
		}else{
			//child
			close(epipe_o[0]);
			close(epipe_e[0]);
			dup2(epipe_o[1],1);
			dup2(epipe_e[1],2);
			sleep(1);
			execvp(args[0],args);
			g_print("Execution failed.");
			_exit(1);
		};
	};
	g_strfreev(args);
	g_free(command);
}



