#include "dialogs.h"
#include "util.h"
#include "callbacks.h"
#include "projects.h"
#include "prefs.h"

//dialogs
gint ask_ok_cancel(gchar *msg)
{
	GtkWidget *dialog = create_questdialog_ok();
	GtkWidget *label = lookup_widget(dialog,"MsgLabel");

	gtk_label_set_text(GTK_LABEL(label),msg);

	return gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
}

gint ask_yes_no_cancel(gchar *msg)
{
	GtkWidget *dialog
 = create_questdialog_yes();
	GtkWidget *label = lookup_widget(dialog,"MsgLabel");

	gtk_label_set_text(GTK_LABEL(label),msg);

	return gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
}

gint ask_yes_no(gchar *msg)
{
	GtkWidget *dialog = gnome_question_dialog_modal_parented(msg,NULL,NULL,GTK_WINDOW(mainapp));

	return gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
}

void ask_error(gchar *msg)
{
	GtkWidget *dialog = gnome_error_dialog_parented(msg,GTK_WINDOW(mainapp));

	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
}


void ask_new_project(gchar *&name,gchar *&dir,gint &type)
{
	GtkWidget *newprojwind = create_newprojdialog();
	GtkWidget *btn, *namelbl, *dirlbl;
	gint answer, itype;
	gchar *iname, *idir, *msg;
	
	name = NULL;
       	dir = NULL;
       	type = 0;

	namelbl = lookup_widget(newprojwind,"entry1");
       	dirlbl = lookup_widget(newprojwind,"fileentry1");
	gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(dirlbl),prefs->defaultdir);
	gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(dirlbl))),prefs->defaultdir);
	
	answer = gnome_dialog_run(GNOME_DIALOG(newprojwind));

       	iname = g_strdup(gtk_entry_get_text(GTK_ENTRY(namelbl)));
       	idir = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(dirlbl),FALSE);

	btn = lookup_widget(newprojwind,"cprojbtn");
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn)))
		itype = 1;
	   		
       	gnome_dialog_close(GNOME_DIALOG(newprojwind));

	if(answer == 1)
		return;

	name = strlen(iname)?g_strdup(iname):NULL;
	
	if(!dir_exists(idir)&&name){
	       	msg = g_strconcat("Directory ",idir, " doesn't exists. Would you like it to be created?",NULL);
	       	answer = ask_yes_no(msg);
       	       	g_free(msg);
	       	if(answer == 0)
	       		dir_create(idir);
	       	else{
	       		g_free(iname);
			g_free(idir);
	       		return;
	       	};
        };

	type = itype;
	dir = idir;

	return;
}	

void ask_open_project(void)
{
	GtkWidget *filew;
         
       	filew = gtk_file_selection_new ("Open project...");
         
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),"clicked",GTK_SIGNAL_FUNC(proj_open_ok),filew);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(filew));
	gtk_widget_show(filew);
}

void ask_saveas_project(void)
{
	GtkWidget *filew;
         
       	filew = gtk_file_selection_new ("Save project as...");
         
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),"clicked",GTK_SIGNAL_FUNC(proj_saveas_ok),filew);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(filew));
	gtk_widget_show(filew);
}


void ask_new_c_file(gint &need_c,gchar *&c_name,gint &need_h,gchar *&h_name)
{
	GtkWidget *dialog = create_newcfiledialog();
	GtkWidget *entry, *toggle;
	gchar *name, *ext;
	gint answer;

	need_c = FALSE;
	need_h = FALSE;
	c_name = NULL;
	h_name = NULL;
	
	answer = gnome_dialog_run(GNOME_DIALOG(dialog));
	if(answer){
		gnome_dialog_close(GNOME_DIALOG(dialog));	
		return;
	};

	toggle = lookup_widget(dialog,"honlycheck");
	need_c = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
	if(need_c){
		entry = lookup_widget(dialog,"cnameentry");
		name = gtk_entry_get_text(GTK_ENTRY(entry));
		entry = lookup_widget(dialog,"extmenu");
		ext = GTK_LABEL(GTK_OPTION_MENU(entry)->button.child)->label;
		c_name = g_strconcat(name,ext,NULL);
	};

	toggle = lookup_widget(dialog,"conlycheck");
	need_h = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
	if(need_h){
		entry = lookup_widget(dialog,"hnameentry");
		name = gtk_entry_get_text(GTK_ENTRY(entry));
		h_name = g_strconcat(name,".h",NULL);
	};

	gnome_dialog_close(GNOME_DIALOG(dialog));
	return;
}

void ask_saveas_file(void)
{
	GtkWidget *filew;
	gchar *dir = g_strconcat(cur_project->get_wdir(),"/",NULL);
         
       	filew = gtk_file_selection_new ("Save file as...");
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew),dir);
         
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),"clicked",GTK_SIGNAL_FUNC(file_saveas_ok),filew);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(filew));
	gtk_widget_show(filew);
}

void ask_add_file(void)
{
	GtkWidget *filew;
	gchar *dir = g_strconcat(cur_project->get_wdir(),"/",NULL);

	filew = gtk_file_selection_new ("Add file to project...");
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew),dir);

	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),"clicked",GTK_SIGNAL_FUNC(add_file_ok),filew);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(filew));
	gtk_widget_show(filew);
}

gchar *ask_remove_file(void)
{
	GtkWidget *dialog = create_removedialog();
	GtkWidget *list = lookup_widget(dialog,"fileslist");
	GtkWidget *lbl;
	gchar *name;

	cur_project->fill_file_list(GTK_LIST(list));

	gint ans = gnome_dialog_run(GNOME_DIALOG(dialog));
	if(ans||!GTK_LIST(list)->selection){
		gnome_dialog_close(GNOME_DIALOG(dialog));
		return NULL;
	};
	
	lbl = GTK_LIST_ITEM(GTK_LIST(list)->selection->data)->item.bin.child;
	gtk_label_get(GTK_LABEL(lbl),&name);
	name = g_strdup(name);
	gnome_dialog_close(GNOME_DIALOG(dialog));
	
	return name;
}

gint ask_line_number(void)
{
	static GtkWidget *dialog;
	if(!dialog)
		dialog = create_gotolinedialog();
	gint ans = gnome_dialog_run(GNOME_DIALOG(dialog));
	gint ln = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(dialog,"linespin")));
	gtk_widget_hide(dialog);
	
	if(!ans)
		return ln;
	else
		return 0;
}





