#include <errno.h>
#include "files.h"
#include "dialogs.h"

gboolean overwrite;

//support func
gboolean compchr(guint16 ch,gpointer cheth)
{
	return ch == (gchar)cheth;
}

//callbacks
void ins_moved_cb(GtkTextView *tkxt,GtkTextViewMovementStep sp,gint count,gboolean ext, GtkStatusbar *stat)
{
	GtkTextBuffer *buf = gtk_text_view_get_buffer(tkxt);
	GtkTextIter it;
	gchar out[100] = "\0";

	gtk_text_buffer_get_iter_at_mark(buf,&it,"insert");
	
	g_snprintf(out,sizeof(out),"L: %d/%d C: %d  %d/%d %s  <%s>",
		   gtk_text_iter_get_line_number(&it)+1,
		   gtk_text_buffer_get_line_count(buf),
		   gtk_text_iter_get_line_char(&it)+1,
		   gtk_text_iter_get_char_index(&it)+1,
		   gtk_text_buffer_get_char_count(buf),
		   tkxt->overwrite_mode?"Overwrite":"Insert",
		   gtk_text_buffer_modified(buf)?"modified":"");

	overwrite = tkxt->overwrite_mode;

	gtk_statusbar_pop(stat,1);
	gtk_statusbar_push(stat,1,out);
}

void cursor_moved_cb(GtkTextView *tkxt,GdkEvent *ev,GtkStatusbar *stat)
{
	GtkTextBuffer *buf = gtk_text_view_get_buffer(tkxt);
	GtkTextIter it;
	gchar out[100] = "\0";

	gtk_text_buffer_get_iter_at_mark(buf,&it,"insert");
	
	g_snprintf(out,sizeof(out),"L: %d/%d C: %d  %d/%d %s  <%s>",
		   gtk_text_iter_get_line_number(&it)+1,
		   gtk_text_buffer_get_line_count(buf),
		   gtk_text_iter_get_line_char(&it)+1,
		   gtk_text_iter_get_char_index(&it)+1,
		   gtk_text_buffer_get_char_count(buf),
		   tkxt->overwrite_mode?"Overwrite":"Insert",
		   gtk_text_buffer_modified(buf)?"modified":"");

	overwrite = tkxt->overwrite_mode;

	gtk_statusbar_pop(stat,1);
	gtk_statusbar_push(stat,1,out);
}

void modif_changed_cb(GtkTextBuffer *buf,GtkStatusbar *stat)
{
	GtkTextIter it;
	gchar out[100] = "\0";

	gtk_text_buffer_get_iter_at_mark(buf,&it,"insert");
	
	g_snprintf(out,sizeof(out),"L: %d/%d C: %d  %d/%d %s  <%s>",
		   gtk_text_iter_get_line_number(&it)+1,
		   gtk_text_buffer_get_line_count(buf),
		   gtk_text_iter_get_line_char(&it)+1,
		   gtk_text_iter_get_char_index(&it)+1,
		   gtk_text_buffer_get_char_count(buf),
		   overwrite?"Overwrite":"Insert",
		   gtk_text_buffer_modified(buf)?"modified":"");

	gtk_statusbar_pop(stat,1);
	gtk_statusbar_push(stat,1,out);
}

gint click_cb(GtkTextView *tkxt,GdkEvent *ev)
{
	GtkTextBuffer *buf = gtk_text_view_get_buffer(tkxt);

	switch(ev->type){
	case GDK_2BUTTON_PRESS:
		if(((GdkEventButton *)ev)->button != 1)
			break;
		//selecting current word
		GtkTextIter it1,it2;
		
		gtk_text_buffer_get_iter_at_mark(buf,&it1,"insert");
		gtk_text_iter_backward_word_start(&it1);
		gtk_text_buffer_get_iter_at_mark(buf,&it2,"insert");
		gtk_text_iter_forward_word_end(&it2);
		gtk_text_buffer_move_mark(buf,"selection_bound",&it1);
		gtk_text_buffer_move_mark(buf,"insert",&it2);
		gtk_signal_emit_stop_by_name(GTK_OBJECT(tkxt),"button_press_event");
		return TRUE;
		break;
	};

	return FALSE;
}

//CFile class
CFile::CFile(gchar *fname,GtkWidget *nb,HighlightSet *h)
{
	name = g_strdup(fname);
	notebook = nb;

	//creating widgets
	editor = gtk_editor_new();
	
	GtkTextTag *tag = gtk_editor_get_default_tag(editor);
	GtkWidget *sw, *vp;
	GtkWidget *label = gtk_label_new(name);
	
	/* setup the default tag */
	gtk_object_set(GTK_OBJECT(tag),
		       "pixels_above_lines", 0,
		       "pixels_below_lines", 0,
		       "font", "-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*-*",
		       NULL);
	
	vp = gtk_viewport_new(NULL, NULL);
	
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	tkxt = gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(editor));
	gtk_container_add(GTK_CONTAINER(sw), tkxt);
	gtk_container_add(GTK_CONTAINER(vp), sw);
	
	prepare_editor(h);

	window = gtk_vbox_new(FALSE,0);
	stat = gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(window),vp,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(window),stat,FALSE,FALSE,0);

	gtk_signal_connect_after(GTK_OBJECT(tkxt),"move_insert",GTK_SIGNAL_FUNC(ins_moved_cb),stat);
	gtk_signal_connect_after(GTK_OBJECT(tkxt),"key_press_event",GTK_SIGNAL_FUNC(cursor_moved_cb),stat);
	gtk_signal_connect(GTK_OBJECT(tkxt),"button_release_event",GTK_SIGNAL_FUNC(cursor_moved_cb),stat);
	gtk_signal_connect(GTK_OBJECT(editor),"modified_changed",GTK_SIGNAL_FUNC(modif_changed_cb),stat);
	gtk_signal_connect(GTK_OBJECT(editor),"changed",GTK_SIGNAL_FUNC(modif_changed_cb),stat);
	
	gtk_signal_connect(GTK_OBJECT(tkxt),"button_press_event",GTK_SIGNAL_FUNC(click_cb),stat);
	
	gtk_widget_show(vp);
	gtk_widget_show(sw);
	gtk_widget_show(label);
	gtk_widget_show(tkxt);
	gtk_widget_show(stat);
	gtk_widget_show(window);
	
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),window,label);

	while(gtk_events_pending())
		gtk_main_iteration();
	
	load();

	cursor_moved_cb(GTK_TEXT_VIEW(tkxt),NULL,GTK_STATUSBAR(stat));
}

CFile::~CFile()
{
	gint page = gtk_notebook_page_num(GTK_NOTEBOOK(notebook),window);
	
	gtk_notebook_remove_page(GTK_NOTEBOOK(notebook),page);
	g_free(name);
}

void CFile::activate(void)
{
	gint page = gtk_notebook_page_num(GTK_NOTEBOOK(notebook),window);
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook),page);
	gtk_widget_grab_focus(tkxt);
}

void CFile::set_name(gchar *newname)
{
	g_free(name);
	name = g_strdup(newname);
	
	gint page = gtk_notebook_page_num(GTK_NOTEBOOK(notebook),window);
	GtkWidget *p = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),page);
	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),p,name);
};

void CFile::prepare_editor(HighlightSet *hl)
{
	GtkTextTag *tag;
    	GtkTextIter start, end;
    	
    	tag = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(editor), "regular");
	
	gtk_text_buffer_get_iter_at_char(GTK_TEXT_BUFFER(editor),&start, 0);
	gtk_text_buffer_get_iter_at_char(GTK_TEXT_BUFFER(editor),
					 &end, gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(editor)));
	gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(editor),"regular", &start, &end);
}

void CFile::highlight(void)
{
	gtk_editor_hilite_buffer(editor);
}

gint CFile::save(void)
{
	FILE *f;
	gchar *text;
	gint len;

	f = fopen(name,"w");
	if(!f){
		len = errno;
		gchar *msg = g_strconcat("Saving file ",name," failed: ",g_strerror(errno),NULL);
		ask_error(msg);
		
		g_free(msg);
		return len;
	};

	len = gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(editor))-1;
	text = gtk_text_buffer_get_text_chars (GTK_TEXT_BUFFER(editor), 0, len, FALSE);
	fputs(text,f);
	g_free(text);
	fclose(f);

	gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(editor),FALSE);
}

gint CFile::save_as(gchar *newname)
{
	gint res;
	gchar *oldname = name;
	GtkWidget *p;

	name = g_strdup(newname);
	res = save();

	if(res){
		g_free(name);
		name = oldname;
	};

	gint page = gtk_notebook_page_num(GTK_NOTEBOOK(notebook),window);
	p = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),page);
	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),p,name);
	
	return res;
}

void CFile::load(void)
{
	FILE *fdata;
	gchar buf[378];
	GtkTextIter it;

	fdata = fopen(name,"r");
	
	while(!feof(fdata)){
		if(!fgets(buf,sizeof(buf),fdata))
			break;
		gtk_text_buffer_insert_at_char(GTK_TEXT_BUFFER(editor),
					       gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(editor))-1,
					       buf, strlen(buf));
	};

	fclose(fdata);

	gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(editor),FALSE);
	gtk_text_buffer_get_iter_at_char(GTK_TEXT_BUFFER(editor),&it,0);
	gtk_text_buffer_place_cursor(GTK_TEXT_BUFFER(editor),&it);
//	highlight();
}

void CFile::cut(void)
{
	gtk_text_buffer_cut(GTK_TEXT_BUFFER(editor),0);
}

void CFile::copy(void)
{
	gtk_text_buffer_copy(GTK_TEXT_BUFFER(editor),0);
}

void CFile::paste(void)
{
	gtk_text_buffer_paste_clipboard(GTK_TEXT_BUFFER(editor),0);
}

void CFile::clear(void)
{
	gtk_text_buffer_delete_selection(GTK_TEXT_BUFFER(editor));
}

void CFile::insert(gchar *text)
{
	gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(editor),text,strlen(text));
}

void CFile::insert(gint after_line,gchar *text)
{
	gtk_text_buffer_insert_after_line(GTK_TEXT_BUFFER(editor),after_line,text,strlen(text));
}

void CFile::gotoline(gint linen)
{
	GtkTextIter it;
	GtkTextMark* mark;
	
	gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(editor),&it,linen-1);
	gtk_text_buffer_place_cursor(GTK_TEXT_BUFFER(editor),&it);
	mark = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(editor),"line",&it,TRUE);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(tkxt),"line",TRUE);
	gtk_text_buffer_delete_mark(GTK_TEXT_BUFFER(editor),"line");
}

//CCFile class

CCFile::CCFile(gchar *fname,GtkWidget *nb,HighlightSet *h)
{
	name = g_strdup(fname);
	notebook = nb;

	//creating widgets
	editor = gtk_editor_new();
	
	GtkTextTag *tag = gtk_editor_get_default_tag(editor);
	GtkWidget *sw, *vp;
	GtkWidget *label = gtk_label_new(name);
	
	/* setup the default tag */
	gtk_object_set(GTK_OBJECT(tag),
		       "pixels_above_lines", 0,
		       "pixels_below_lines", 0,
		       "font", "-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*-*",
		       NULL);
	
	vp = gtk_viewport_new(NULL, NULL);
	
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	tkxt = gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(editor));
	gtk_container_add(GTK_CONTAINER(sw), tkxt);
	gtk_container_add(GTK_CONTAINER(vp), sw);
	
	prepare_editor(h);

	window = gtk_vbox_new(FALSE,0);
	stat = gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(window),vp,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(window),stat,FALSE,FALSE,0);

	gtk_signal_connect_after(GTK_OBJECT(tkxt),"move_insert",GTK_SIGNAL_FUNC(ins_moved_cb),stat);
	gtk_signal_connect_after(GTK_OBJECT(tkxt),"key_press_event",GTK_SIGNAL_FUNC(cursor_moved_cb),stat);
	gtk_signal_connect(GTK_OBJECT(tkxt),"button_release_event",GTK_SIGNAL_FUNC(cursor_moved_cb),stat);
	gtk_signal_connect(GTK_OBJECT(editor),"modified_changed",GTK_SIGNAL_FUNC(modif_changed_cb),stat);
	gtk_signal_connect(GTK_OBJECT(editor),"changed",GTK_SIGNAL_FUNC(modif_changed_cb),stat);
	
	gtk_signal_connect(GTK_OBJECT(tkxt),"button_press_event",GTK_SIGNAL_FUNC(click_cb),stat);
	
	gtk_widget_show(vp);
	gtk_widget_show(sw);
	gtk_widget_show(label);
	gtk_widget_show(tkxt);
	gtk_widget_show(stat);
	gtk_widget_show(window);
	
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),window,label);

	while(gtk_events_pending())
		gtk_main_iteration();
	
	load();

	cursor_moved_cb(GTK_TEXT_VIEW(tkxt),NULL,GTK_STATUSBAR(stat));
}

void CCFile::prepare_editor(HighlightSet *hl)
{
	 GtkTextTag *tag;
	 GtkTextIter start, end;
	
	if(!hl->scanner)
		return;
		
	gtk_editor_install_scanner(editor,hl->scanner);
  
	tag = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(editor), "regular");
  

	tag = gtk_editor_create_tag(editor, "keyword");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "blue",
		       "font", "-adobe-courier-bold-r-normal-*-12-*-*-*-*-*-*-*",
		       NULL);

	tag = gtk_editor_create_tag(editor, "type");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "#00d81c",
		       NULL);
	
	tag = gtk_editor_create_tag(editor, "include");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "#8700bc",
		       NULL);
	
	tag = gtk_editor_create_tag(editor, "pre_processor");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "#8700bc",
		       NULL);
	
	tag = gtk_editor_create_tag(editor, "comment");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "#ff0000",
		       "font", "-adobe-courier-medium-i-normal-*-12-*-*-*-*-*-*-*",
		       NULL);
		    
	tag = gtk_editor_create_tag(editor, "cppcomment");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "#ff0000",
		       "font", "-adobe-courier-medium-i-normal-*-12-*-*-*-*-*-*-*",
		       NULL);	       
	
	tag = gtk_editor_create_tag(editor, "string");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "#d0b800",
		       NULL);
	
	tag = gtk_editor_create_tag(editor, "escape_char");
	gtk_object_set(GTK_OBJECT(tag),
		       "foreground", "#4ad822",
		       NULL);
  

	tag = gtk_editor_create_tag(editor, "function");
	gtk_object_set(GTK_OBJECT(tag),
                 "foreground", "blue",
		       NULL);
		       
	tag = gtk_editor_create_tag(editor, "classname");
	gtk_object_set(GTK_OBJECT(tag),
                 "foreground", "#c000bb",
		 "font", "-adobe-courier-bold-r-normal-*-12-*-*-*-*-*-*-*",
		       NULL);	       
	
	gtk_text_buffer_get_iter_at_char(GTK_TEXT_BUFFER(editor),
					 &start, 0);
	gtk_text_buffer_get_iter_at_char(GTK_TEXT_BUFFER(editor),
					 &end, gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(editor)));
	gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(editor),
				  "regular", &start, &end);
}

void CCFile::comment_selection(void)
{
	GtkTextBuffer *buf = GTK_TEXT_BUFFER(editor);
	GtkTextIter start,end;
	gint ch, c;

	gtk_text_buffer_get_iter_at_mark(buf,&start,"insert");
	gtk_text_buffer_get_iter_at_mark(buf,&end,"selection_bound");

	if(!gtk_text_iter_equal(&start,&end)){
		c = gtk_text_iter_compare(&start,&end);
		gtk_text_iter_reorder(&start,&end);

		ch = gtk_text_iter_get_char_index(&end);
		
		gtk_text_buffer_insert(buf,&start,"/*",2);
		gtk_text_buffer_get_iter_at_char(buf,&end,ch+2);
		gtk_text_buffer_insert(buf,&end,"*/",2);
		gtk_text_buffer_get_iter_at_char(buf,&end,ch+2);
		if(c)
			gtk_text_buffer_move_mark(buf,"insert",&end);
		else
			gtk_text_buffer_move_mark(buf,"selection_bound",&end);
	};
}

void CCFile::comment_line(void)
{
	GtkTextBuffer *buf = GTK_TEXT_BUFFER(editor);
	GtkTextIter it,it1;
	gint line;

	gtk_text_buffer_get_iter_at_mark(buf,&it,"insert");
	line = gtk_text_iter_get_line_number(&it);

	gtk_text_buffer_get_iter_at_line_char(buf,&it1,line,0);
	gtk_text_buffer_insert(buf,&it1,"//",2);
}

void CCFile::back_statement(void)
{
	GtkTextBuffer *buf = GTK_TEXT_BUFFER(editor);
	GtkTextIter it;

	gtk_text_buffer_get_iter_at_mark(buf,&it,"insert");
	if(gtk_text_iter_backward_find_char(&it,compchr,(void *)';')){
		gtk_text_iter_backward_find_char(&it,compchr,(void *)';');
		gtk_text_iter_forward_chars(&it,2);
		
		gtk_text_buffer_move_mark(buf,"insert",&it);
		gtk_text_buffer_move_mark(buf,"selection_bound",&it);
	};
}

void CCFile::forw_statement(void)
{
	GtkTextBuffer *buf = GTK_TEXT_BUFFER(editor);
	GtkTextIter it;

	gtk_text_buffer_get_iter_at_mark(buf,&it,"insert");
	if(gtk_text_iter_forward_find_char(&it,compchr,(void *)';')){
		gtk_text_iter_forward_chars(&it,2);
		
		gtk_text_buffer_move_mark(buf,"insert",&it);
		gtk_text_buffer_move_mark(buf,"selection_bound",&it);
	};
}

























