#include "prefs.h"
#include "common.h"
#include "util.h"
#include "support.h"
#include "projects.h"

Preferences *prefs;

Preferences::Preferences(void)
{
	file = g_strconcat(home,"/.titano/titano",NULL);
	
	if(Preferences::check())
		load();
	defaults();
	save();
}

Preferences::Preferences(const Preferences &pr)
{
	file = g_strconcat(home,"/.titano/titano",NULL);

	name = g_strdup(pr.name);
	mail = g_strdup(pr.mail);
	defaultdir = g_strdup(pr.defaultdir);
	lastname = g_strdup(pr.lastname);
	lastdir = g_strdup(pr.lastdir);
	
	def_cppcmd.make = g_strdup(pr.def_cppcmd.make);
	def_cppcmd.compile = g_strdup(pr.def_cppcmd.compile);
	def_cppcmd.debug = g_strdup(pr.def_cppcmd.debug);
	def_cppcmd.execute = g_strdup(pr.def_cppcmd.execute);
}

Preferences::~Preferences()
{
	g_free(name);
	g_free(mail);
	g_free(defaultdir);
	g_free(lastname);
	g_free(lastdir);
	
	g_free(def_cppcmd.make);
	g_free(def_cppcmd.compile);
	g_free(def_cppcmd.debug);
	g_free(def_cppcmd.execute);

	g_free(file);
}

gboolean Preferences::check(void)
{
	gchar *ifile = g_strconcat(home,"/.titano/titano",NULL);
	gboolean a =  file_exists(ifile);
	g_free(ifile);

	return a;
}

void Preferences::defaults(void)
{
	if(!name)
		name = g_getenv("USER");
	
	if(!mail){
		gchar *host = g_strdup(g_getenv("HOSTNAME"));
		mail = g_strconcat(name,"@",host,NULL);
		g_free(host);
	};

	if(!defaultdir)
		defaultdir = g_strconcat(home,"/projects",NULL);

	if(!def_cppcmd.make)
		def_cppcmd.make = g_strdup("make");

	if(!def_cppcmd.compile)
		def_cppcmd.compile = g_strdup("gcc -g $FFNAME -o $FNAME 2>&1");

	if(!def_cppcmd.debug)
		def_cppcmd.debug = g_strdup("xterm -e gdb $NAME");

	if(!def_cppcmd.execute)
		def_cppcmd.execute = g_strdup("./$NAME");
}

void Preferences::load(void)
{
	read_conf(file,"General/defdir",defaultdir);
	read_conf(file,"General/lastname",lastname);
	read_conf(file,"General/lastdir",lastdir);

	read_conf(file,"User/name",name);
	read_conf(file,"User/mail",mail);

	read_conf(file,"Cpp Commands/make",def_cppcmd.make);
	read_conf(file,"Cpp Commands/compile",def_cppcmd.compile);
	read_conf(file,"Cpp Commands/debug",def_cppcmd.debug);
	read_conf(file,"Cpp Commands/execute",def_cppcmd.execute);
}

void Preferences::load(GtkWidget *prbox)
{
	GtkWidget *entry;

	entry = lookup_widget(prbox,"defdir");
	g_free(defaultdir);
	defaultdir = g_strdup(gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(entry),TRUE));
	
	entry = lookup_widget(prbox,"username");
	g_free(name);
	name = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

	entry = lookup_widget(prbox,"usermail");
	g_free(mail);
	mail = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

	entry = lookup_widget(prbox,"cppmake");
	g_free(def_cppcmd.make);
	def_cppcmd.make = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

	entry = lookup_widget(prbox,"cppcomp");
	g_free(def_cppcmd.compile);
	def_cppcmd.compile = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

	entry = lookup_widget(prbox,"cppdebug");
	g_free(def_cppcmd.debug);
	def_cppcmd.debug = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

	entry = lookup_widget(prbox,"cppexec");
	g_free(def_cppcmd.execute);
	def_cppcmd.execute = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void Preferences::save(void)
{
	write_conf(file,"General/defdir",defaultdir);
	if(cur_project){
		write_conf(file,"General/lastname",cur_project->get_name());
		write_conf(file,"General/lastdir",cur_project->get_dir());
	}else{
		if(lastname){
			write_conf(file,"General/lastname",lastname);
			write_conf(file,"General/lastdir",lastdir);
		}else{
			write_conf(file,"General/lastname",(gchar *)NULL);
			write_conf(file,"General/lastdir",(gchar *)NULL);
		};
	};
	
	
	write_conf(file,"User/name",name);
	write_conf(file,"User/mail",mail);
	
	write_conf(file,"Cpp Commands/make",def_cppcmd.make);
	write_conf(file,"Cpp Commands/compile",def_cppcmd.compile);
	write_conf(file,"Cpp Commands/debug",def_cppcmd.debug);
	write_conf(file,"Cpp Commands/execute",def_cppcmd.execute);

	drop_conf(file);
}

void Preferences::save(GtkWidget *prbox)
{
	GtkWidget *entry;

	entry = lookup_widget(prbox,"defdir");
	gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(entry),defaultdir);
	gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(entry))),defaultdir);
	
	entry = lookup_widget(prbox,"username");
	gtk_entry_set_text(GTK_ENTRY(entry),name);

	entry = lookup_widget(prbox,"usermail");
	gtk_entry_set_text(GTK_ENTRY(entry),mail);

	entry = lookup_widget(prbox,"cppmake");
	gtk_entry_set_text(GTK_ENTRY(entry),def_cppcmd.make);

	entry = lookup_widget(prbox,"cppcomp");
	gtk_entry_set_text(GTK_ENTRY(entry),def_cppcmd.compile);

	entry = lookup_widget(prbox,"cppdebug");
	gtk_entry_set_text(GTK_ENTRY(entry),def_cppcmd.debug);

	entry = lookup_widget(prbox,"cppexec");
	gtk_entry_set_text(GTK_ENTRY(entry),def_cppcmd.execute);
}






