#include "projects.h"
#include "dialogs.h"
#include "util.h"

extern void up_iterface(void);

CProject *cur_project = NULL;

//routines for glib

gint strcomp(const void *a,const void *b)
{
	return g_strcasecmp((gchar *)a,(gchar *)b);
}

gint filescmp(const void *a,const void *b)
{
	return g_strcasecmp((gchar *)b,((CFile *)a)->get_name());
}

//needed callbacks

void files_chg_cb(GtkNotebook *nb,gpointer data,gint n,gpointer udata)
{
	GtkWidget *page, *label;
	gchar *name;

	if(!cur_project||(n == -1))
		return;
	page = gtk_notebook_get_nth_page(nb,n);
	label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(nb),page);
	gtk_label_get(GTK_LABEL(label),&name);

	cur_project->set_file(name);
}

void notebook_child_focus_cb(GtkNotebook *nb,gpointer data)
{
	GtkWidget *page, *label;
	gchar *name;
  	gint n;

	if(!cur_project)
		return;
  	n = gtk_notebook_get_current_page(nb);
	if(n == -1)
		return;
	page = gtk_notebook_get_nth_page(nb,n);
	label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(nb),page);
	gtk_label_get(GTK_LABEL(label),&name);

	cur_project->set_file(name);
}

GtkDirectionType notebook_focus_cb(GtkNotebook *nb,GtkDirectionType dir,gpointer data)
{
	GtkWidget *page, *label;
	gchar *name;
  	gint n;

	if(!cur_project)
		return dir;
  	n = gtk_notebook_get_current_page(nb);
	if(n == -1)
		return dir;
	page = gtk_notebook_get_nth_page(nb,n);
	label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(nb),page);
	gtk_label_get(GTK_LABEL(label),&name);

	cur_project->set_file(name);
}

void item_selected(GtkItem *item,gchar *name)
{
	if(!cur_project->file_is_opened(name))
		cur_project->open_file(name);
	else
		cur_project->activate_file(name);

	up_interface();
}

//dynamic menus callbacks
void on_make_activate(GtkMenuItem *item,gpointer data)
{
	if(cur_project->has_modif_files()){
		gint ans = ask_yes_no_cancel("You are going to make your project. It has modified files. Save them all?");	
		switch(ans){
		case 0:cur_project->save_all_files();
			break;
		case 3:return;
		};
	};
	
	cur_project->make();
}

void on_compile_activate(GtkMenuItem *item,gpointer data)
{
	if(!cur_project->cur_file)
		return;
	if(cur_project->cur_file->modified()){
		gint ans = ask_yes_no_cancel("You are going to compile file. It was modified. Save?");	
		switch(ans){
		case 0:cur_project->cur_file->save();
			break;
		case 3:return;
		};
	};

	cur_project->compile();
}

void on_debug_activate(GtkMenuItem *item,gpointer data)
{
	cur_project->debug();
}

void on_execute_activate(GtkMenuItem *item,gpointer data)
{
	cur_project->execute();
}

void on_comsel_activate(GtkMenuItem *item,gpointer data)
{
	cur_project->cur_file->comment_selection();
}

void on_comline_activate(GtkMenuItem *item,gpointer data)
  {
	cur_project->cur_file->comment_line();
}

void on_back_activate(GtkMenuItem *item,gpointer data)
{
	cur_project->cur_file->back_statement();
}

void on_forw_activate(GtkMenuItem *item,gpointer data)
{
	cur_project->cur_file->forw_statement();
}

//CProject class

gchar *CProject::form_name(gchar *fullname)
{
	gchar **splfullname = NULL;
	gchar **splwd = NULL;
	gchar *fn[127], *file, *wd;
	gint i,j,k;
	i = 1;
	j = 0;

	splwd = g_strsplit(wdir,"/",127);
	splfullname = g_strsplit(fullname,"/",127);
	while(splfullname[i]&&splwd[i]&&!g_strcasecmp(splfullname[i],splwd[i])){
		i++;
	};

	k = i;
	while(splwd[i++])
		fn[j++] = g_strdup("..");
	while(splfullname[k])
		fn[j++] = g_strdup(splfullname[k++]);
	fn[j] = NULL;

	g_strfreev(splfullname);
	g_strfreev(splwd);

	file = g_strjoinv("/",fn);
	for(i = 0;i <= j;i++)
		g_free(fn[i]);

	return file;
}

gchar *CProject::prepare_command(gchar *cmd)
{
	gchar *res, *res1;
	gchar *fname, *ffname;
	
	res1 = str_replace(cmd,"$NAME",name);
	
	if(cur_file){
		fname = cur_file->get_name();
		res = str_replace(res1,"$FNAME",fname);
		g_free(res1);
		ffname = g_new(gchar,strchr(fname,'.')-fname+1);
		g_snprintf(ffname,strchr(fname,'.')-fname+1,"%s",fname);
		res1 = str_replace(res,"$FFNAME",ffname);
		g_free(res);
    
		g_free(ffname);
	};

	res = res1;
	return res;
}
