#ifndef _PROJECTS_H
#define _PROJECTS_H

#include <typeinfo>

#include "common.h"
#include "files.h"

extern gchar libdir[];

//exported glib routines

gint strcomp(const void *a,const void *b);
gint filescmp(const void *a,const void *b);

//exported callbacks

void files_chg_cb(GtkNotebook *nb,gpointer data,gint n,gpointer udata);
void notebook_child_focus_cb(GtkNotebook *nb,gpointer data);
GtkDirectionType notebook_focus_cb(GtkNotebook *nb,GtkDirectionType dir,gpointer data);
void item_selected(GtkItem *item,gchar *name);

//dynamic menus callbacks

void on_make_activate(GtkMenuItem *item,gpointer data);
void on_compile_activate(GtkMenuItem *item,gpointer data);
void on_debug_activate(GtkMenuItem *item,gpointer data);
void on_execute_activate(GtkMenuItem *item,gpointer data);

void on_comsel_activate(GtkMenuItem *item,gpointer data);
void on_comline_activate(GtkMenuItem *item,gpointer data);
void on_back_activate(GtkMenuItem *item,gpointer data);
void on_forw_activate(GtkMenuItem *item,gpointer data);

//classes
class CProject
{
  protected:
	gchar *name;
	gchar *dir, *wdir;
	GtkWidget *app;
	GtkWidget *window;
	GtkWidget *compile_list;
	GtkWidget *stdout_list;
	GtkWidget *stderr_list;
	HighlightSet *hilite;

	GtkWidget *otree;
	GtkWidget *oitem;
	
	gchar *cmake, *comp, *deb, *exec;
	gint make_id, comp_id, exec_out_id, exec_err_id;
	FILE *make_f, *comp_f;
	
	void virtual load(void) = 0;

	void virtual create_menus(void) = 0;
	void virtual destroy_menus(void) = 0;

	gchar *prepare_command(gchar *cmd);
  public:
	CFile *cur_file;
	
	virtual ~CProject() { };
	gchar *form_name(gchar *fullname);

	gchar *get_name(void) { return name; };
	gchar *get_dir(void) { return dir; };
	gchar *get_wdir(void) { return wdir; };
	
	void virtual load(GtkWidget *propbox) = 0;
	void virtual save(void) = 0;
	void virtual save(GtkWidget *propbox) = 0;
	void virtual save_as(gchar *newname) = 0;
	
	gboolean can_make(void) { return cmake != NULL; };
	gboolean can_compile(void) { return comp != NULL; };
	gboolean can_debug(void) { return deb != NULL; };
	gboolean can_execute(void) { return exec != NULL; };

	void virtual new_file(gchar *name) = 0;
	void virtual open_file(gchar *name) = 0;
	void virtual save_file_as(gchar *newname) = 0;
	void virtual close_file(void) = 0;
	void virtual close_file(gchar *name) = 0;
	void virtual activate_file(gchar *name) = 0;
	void virtual add_file(gchar *name) = 0;
	void virtual remove_file(gchar *name) = 0;	
	void virtual save_all_files(void) = 0;
	gboolean virtual has_modif_files(void) = 0;
	void virtual fill_file_list(GtkList *list) = 0;

	void virtual make(void) { };
	void virtual compile(void) { };
	void virtual debug(void) { };
	void virtual execute(void) { };
	
	void virtual set_title(void) = 0;
	void virtual load_highlights(void) = 0;
	gboolean virtual file_in_project(gchar *name) = 0;
	gboolean virtual file_is_opened(gchar *name) = 0;
        void virtual set_file(gchar *name) = 0;

	friend void make_cb(gint *pid,gint file,GdkInputCondition cond);
	friend void compile_cb(gint *pid,gint file,GdkInputCondition cond);
	friend void exec_out_cb(gint *pid,gint file,GdkInputCondition cond);
	friend void exec_err_cb(gint *pid,gint file,GdkInputCondition cond);
};

extern CProject *cur_project;

class CCppProject : public CProject
{
  private:
	GtkWidget *hpaned1, *hpaned2;
	GtkWidget *projtree, *htree, *ctree;
	GtkWidget *hitem, *citem;
	GtkWidget *cnotebook, *hnotebook;
	
	GSList *cfiles;
	gint cfiles_cnt;
	GSList *hfiles;
	gint hfiles_cnt;
	GSList *open_cfiles;
	gint open_c_cnt;
	GSList *open_hfiles;
	gint open_h_cnt;
	
	void load(void);
	
	void virtual create_menus(void);
	void virtual destroy_menus(void);
	
	CHFile *get_h_file(gchar *name);
	CCFile *get_c_file(gchar *name);
  public:
	CCppProject(gchar *pname,gchar* d,GtkWidget *a,GtkWidget *vpaned,GtkWidget *clist,GtkWidget *olist,GtkWidget *elist);
	virtual ~CCppProject();
	
	static gint create_empty(gchar *name,gchar *dir,gchar *def_make,gchar *def_compile,gchar *def_debug,gchar *def_exec);
	
	void virtual set_title(void);

	void virtual load(GtkWidget *propbox);
	void virtual save(void);
	void virtual save(GtkWidget *propbox);
	void virtual save_as(gchar *newname);
	static gboolean can_translate_format(gint form);
	
	void virtual new_file(gchar *name);
	void virtual open_file(gchar *name);
	void virtual save_file_as(gchar *newname);
	void virtual close_file(void);
	void virtual close_file(gchar *name);
	void virtual activate_file(gchar *name);
	void virtual add_file(gchar *name);
	void virtual remove_file(gchar *name);
	void virtual save_all_files(void);
	gboolean virtual has_modif_files(void);
	void virtual fill_file_list(GtkList *list);

	void virtual make(void);
	void virtual compile(void);
	void virtual debug(void);
	void virtual execute(void);
	
	void virtual load_highlights(void);
	gboolean virtual file_in_project(gchar *name);
	gboolean virtual file_is_opened(gchar *name);
	void virtual set_file(gchar *name);
};

#endif




