#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gnome.h>
#include "util.h"

gint dir_exists(gchar *dir)
{
	gchar *curdir = NULL;
	gint res;

	curdir = getcwd(curdir,0);
	res = (chdir(dir) == 0);
	chdir(curdir);
	g_free(curdir);

	return res;
}

gint dir_create(gchar *name)
{
	gchar *nname = g_strconcat("mkdir  ",name,"/",NULL);
	gint dir;

	dir =system(nname);
	g_free(nname);
	if(dir == -1){
		g_error("Dir create failed: ");
		return 0;
	};

	return 1;
}

gint file_exists(gchar *file)
{
	gint f,res;

	f = open(file,O_RDONLY);
	res = (f > 0);
	close(f);
	
	return res;
}

gint file_create(gchar *file)
{
	FILE *f;

	f = fopen(file,"w");
	if(!f){
		g_error("File create failed: ");
		return 0;
	};
	fclose(f);

	return 1;
}

gchar *fullname(gchar *dir, gchar *file)
{
	gchar *f, *d;
	gchar dirc[384], filec[384];
	g_snprintf(dirc,sizeof(dirc),"%s",dir);
	g_snprintf(filec,sizeof(filec),"%s",file);
	
	d = dirc;
	f = filec;

	while(strstr(f,"..")){
		f += 3;
		*(strrchr(d,'/')) = '\0';
	};
	
	return g_strconcat(d,"/",f,NULL);
}

void drop_conf(gchar *file)
{
	gchar *prefix = g_strconcat("=",file,"=",NULL);
	
	gnome_config_sync_file(prefix);
	gnome_config_drop_file(prefix);
	g_free(prefix);
}

void clear_sect_conf(gchar *file,gchar *section)
{
	gchar *prefix = g_strconcat("=",file,"=",section,NULL);
	
	gnome_config_clean_section(prefix);
	g_free(prefix);
}


void clear_key_conf(gchar *file,gchar *key)
{
	gchar *prefix = g_strconcat("=",file,"=",key,NULL);
	
	gnome_config_clean_key(prefix);
	g_free(prefix);
}

void write_conf(gchar *file,gchar *path,gchar *value)
{
	gchar *prefix = g_strconcat("=",file,"=",path,NULL);

	gnome_config_set_string(prefix,value);
	g_free(prefix);
}

void write_conf(gchar *file,gchar *path,gint value)
{
	gchar *prefix = g_strconcat("=",file,"=",path,NULL);

	gnome_config_set_int(prefix,value);
	g_free(prefix);
}

void write_conf_bool(gchar *file,gchar *path,gboolean value)
{
	gchar *prefix = g_strconcat("=",file,"=",path,NULL);

	gnome_config_set_bool(prefix,value);
	g_free(prefix);
}

void read_conf(gchar *file,gchar *path,gchar *&value)
{
	gchar *prefix = g_strconcat("=",file,"=",path,NULL);

	value = gnome_config_get_string(prefix);
	g_free(prefix);
}

void read_conf(gchar *file,gchar *path,gint &value)
{
	gchar *prefix = g_strconcat("=",file,"=",path,NULL);

	value = gnome_config_get_int(prefix);
	g_free(prefix);
}

void read_conf_bool(gchar *file,gchar *path,gboolean &value)
{
	gchar *prefix = g_strconcat("=",file,"=",path,NULL);

	value = gnome_config_get_bool(prefix);
	g_free(prefix);
}

gchar *str_replace(gchar *str,gchar *rem,gchar *ins)
{
	gchar *str1 = g_strconcat(str," ",NULL);
	gchar *res1 = NULL, *res = NULL;
	gchar **array = g_strsplit(str1,rem,127);
	g_free(str1);
	
	if(!array[1])
		res = g_strdup(str);
	else{
		res1 = g_strjoinv(ins,array);
		if(res1[strlen(res1)-1] == ' ')
			res = g_strndup(res1,strlen(res1)-1);
		else
			res = res1;	
		g_free(res1);	
	};
		
	g_strfreev(array);
	return res;
}
